;*********************************************************************
; Filename:    16F628_Bremslicht.ASM
; Author:      GroberKlotz
; Created:     Datum
; Copyright:   free for non commercial use
;
;*********************************************************************
; Hardware:    PIC16F628A
;              Interner Oszillator fosc=4MHz
;              PORTA       nicht verwendet
;              PORTB,RB1   Rote LED mit Rv 1k an GND
;
; Info: Es ist sinnvoller die LED mit Vorwiderstand zwischen Pin und
;       Vdd (+5V) zu schalten, da die Ausgangsstufe am PIN 
;       aufbaubedingt (in elektrischer Hinsicht) besser Strom 
;       aufnehmen als abgeben kann.
;*********************************************************************
; PROJEKT
;     Simulation eines Bremslicht das zunaechst mit einer Frequenz von
;     ca. 5Hz (0,1s ein, 0,1s aus) 3x blinkt und dann fuer 2sec
;     dauernd leuchtet. Nach einer Pause von 5s beginnt der Vorgang
;     wieder von vorne.
;  
;*********************************************************************
;--SUPPRESSED WARNINGS (nur wenn Code fehlerfrei ist auskommentieren)
   ERRORLEVEL -302   ;"Register in operand not in bank 0. Ensure .."
   ERRORLEVEL -207   ;"Found label after column 1."
   ;
;--MCU-KONFIGURATION             (MCU=MicroControllerUnit)
   LIST     P=16F628A           ;directive to define then processor
 #include "p16F628A.inc"       ;processor specific variable definitions
; CONFIG
; __config 0x3F70
 __CONFIG _FOSC_INTOSCIO & _WDTE_OFF & _PWRTE_ON & _MCLRE_ON & _BOREN_ON & _LVP_OFF & _CPD_OFF & _CP_OFF
;  INFO ZU _FOSC_INTOSCIO: Es wird der interne Oszillator des 16F628
;  mit der Default-Frequenz = 4MHz verwendet. Dadurch sind die Pins 
;  RA7:6 als I/O verwendbar und der Quarz entfaellt.
   ;
;*********************************************************************
;--VARIABLES DEFINITIONS
   CBLOCK   0x20              ;Variablen beginnen ab Adresse 0x20
      d1,d2,d3                ;Zaehlvariable in Sub Delay100ms 
      Blink_Mult             ;Multiplikator fuer Basis-Delay 100ms
   ENDC
   ;
;--LABELS & CONSTANTS
   LED_1    EQU      RB1                  ;Label Wert = RB1 = 1
   #DEFINE  LED_ON   bsf   PORTB,LED_1    ;schaltet LED-Pin auf H-Pegel
   #DEFINE  LED_OFF  bcf   PORTB,LED_1    ;Schaltet LED-Pin auf L-Pegel
   ;
;*********************************************************************
   ORG      0x0000            ; Programmstart nach Reset
   GOTO     Init              ; Ports u.a. initialisieren
;*********************************************************************
   ORG      0x0004            ; interrupt vector
   RETFIE						   ; wird hier nicht verwendet
;*********************************************************************
Init:
;--PORTA                      ;S.31 Pin Out description
   BANKSEL  PORTA             ;wechselt zu PORTA-Register in Bank 0
   clrf     PORTA             ;clear output latches
   movlw    0x007             ;schaltet analog pins als digital I/O
   movwf    CMCON
   BANKSEL  TRISA             ;Datenrichtungsregister liegt in Bank1
   ;TRISA<5> always read as '1' TRISA<7:6>depend on oscillatormmode
   clrf     TRISA             ;RA7:6,4:0 outputs, RA5 Reset-Pin
   ;   
;--PORTB
   BANKSEL  PORTB             ;wechselt zu PORT-Register in Bank 0
   clrf     PORTB             ;clear output latches
   BANKSEL  TRISB             ;Datenrichtungsregister liegt in Bank1
   clrf     TRISB             ;Set RB<7:0> as outputs
   BANKSEL  0                 ;Zurueck nach Bank 0
   ;
   GOTO     Main              ;entbehrlich
;*********************************************************************
;  HAUPTPROGRAMM
;*********************************************************************
Main
   movlw    .1                ;Wert fuer Delayzeit = 1x100ms = 0,1s
   movwf    Blink_Mult        ;Multiplikator fuer Delay Blinken
; Subroutinen sollten flexibel sein, daher wird die Delayzeit als
; Multiplikator fuer das Basis-Delay im WREG uebergeben.
   ;
Main_Loop
;  LED an RB1 blinkt ab hier 3x 
   CALL  Blink_LED            ;LED an RB1 0,1s EIN -> 0,1s AUS
   CALL  Blink_LED
   CALL  Blink_LED
   ;
;  LED an RB1 wird fuer 2s ununterbrochen eingeschaltet
   LED_ON                     ;LED einschalten
   movlw d'20'                ;Wert fuer Delayzeit 20x100ms=2s
   CALL  Delay100ms           ;LED leuchtet waehrend 2s
   ;
;  LED wird fuer 5s ausgeschaltet - Danach beginnt der Zyklus
;  Blinken - 2s ein von Neuem.
   LED_OFF                    ;LED an RB1 aus
   movlw d'50'                ;Wert fuer Delayzeit = 50x100ms = 5s
   CALL  Delay100ms           ;LED bleibt 5s ausgeschaltet
   ;
   GOTO  Main_Loop            ;zum Schleifenanfang
;****************************************************************
;  UNTERPROGRAMME
;****************************************************************
Blink_LED
; Eine  LED wird ein- und dann wieder ausgeschaltet. Die Leucht- bzw
; Dunkelzeit dauert gleich lange und wird durch den im Registers
; Blink_Mult enthaltenen Wert als Uebergabeparameter, zusammen mit 
; der Sub "Delay100ms" bestimmt.
   ;
   LED_ON                     ;Label enthaelt: bsf PORTB,RB1 (RB1=1)
   movf  Blink_Mult,w         ;      LED EIN
   CALL  Delay100ms
   ;
   LED_OFF                    ;Label enthaelt: bcf PORTB,RB1 (RB1=0)
   movf  Blink_Mult,w         ;      LED AUS
   CALL  Delay100ms
   RETURN   
;****************************************************************
Delay100ms              ;bei fosc=4MHz
; bergabe Anzahl 1/10s im WREG
;
;  Beispiel: movlw   d'10"
;            CALL    Delay100ms  ;Delaytime=1s (10x100ms)
;
; d3=100 -> 10.000.008 instruction cycles incl. CALL
   movwf    d3          ;falls d3=0, Delayzeit = 100ms vorgeben:
   xorlw    .0          ;Vergleich WREG = d3 = 0 
   btfsc    STATUS,Z    ;Z=1? WREG=0? 
   incf     d3,f        ;JA, d3+1 -> 100ms Delayzeit
Delay100ms_0:
   nop
   movlw    0x1E
   movwf    d1
   movlw    0x4F
   movwf    d2
Delay100ms_1:
   decfsz   d1, f
   GOTO     $+2
   decfsz   d2, f
   GOTO     Delay100ms_1
   GOTO     $+1
   nop
   decfsz   d3,f
   GOTO     Delay100ms_0
   nop
   RETURN   ;Generated by http://www.piclist.com/cgi-bin/delay.exe*
;******************************************************************
   END